﻿using Microsoft.IdentityModel.Protocols.OpenIdConnect;
using Microsoft.IdentityModel.Tokens;
using Microsoft.Owin;
using Microsoft.Owin.Security;
using Microsoft.Owin.Security.Cookies;
using Microsoft.Owin.Security.Notifications;
using Microsoft.Owin.Security.OpenIdConnect;
using Owin;
using System.Threading.Tasks;
using System.Web;
using Wisej.Web;

[assembly: OwinStartup(typeof(Wisej.OpenID.Program))]

namespace Wisej.OpenID
{
	class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		static void Main()
		{
			Application.MainPage = new Page1();
		}

		//
		// You can use the entry method below
		// to receive the parameters from the URL in the args collection.
		//
		//static void Main(NameValueCollection args)
		//{
		//}

		// client id from the azure AD application.
		string clientId = "CLIENT ID";

		// tenant id from the azure AD application.
		static string tenant = "TENANT ID";

		// authority is the URL for authority, composed of the Microsoft identity platform and the tenant name (e.g. https://login.microsoftonline.com/contoso.onmicrosoft.com/v2.0)
		string authority = $"https://login.microsoftonline.com/{tenant}/v2.0";

		// redirectUri is the URL where the user will be redirected to after they sign in.
		string redirectUri = "https://localhost:44351/authenticate/";

		public void Configuration(IAppBuilder app)
		{
			app.SetDefaultSignInAsAuthenticationType(CookieAuthenticationDefaults.AuthenticationType);

			app.UseCookieAuthentication(new CookieAuthenticationOptions());

			app.UseOpenIdConnectAuthentication(
				new OpenIdConnectAuthenticationOptions
				{
					// Sets the ClientId, authority, RedirectUri as obtained from web.config
					ClientId = clientId,
					Authority = authority,
					RedirectUri = redirectUri,
					RequireHttpsMetadata = false,
					// PostLogoutRedirectUri is the page that users will be redirected to after sign-out. In this case, it is using the home page
					PostLogoutRedirectUri = redirectUri,
					Scope = OpenIdConnectScope.OpenIdProfile,
					// ResponseType is set to request the code id_token - which contains basic information about the signed-in user
					ResponseType = OpenIdConnectResponseType.CodeIdToken,
					// ValidateIssuer set to false to allow personal and work accounts from any organization to sign in to your application
					// To only allow users from a single organizations, set ValidateIssuer to true and 'tenant' setting in web.config to the tenant name
					// To allow users from only a list of specific organizations, set ValidateIssuer to true and use ValidIssuers parameter
					TokenValidationParameters = new TokenValidationParameters()
					{
						ValidateIssuer = false // This is a simplification
					},
					// OpenIdConnectAuthenticationNotifications configures OWIN to send notification of failed authentications to OnAuthenticationFailed method
					Notifications = new OpenIdConnectAuthenticationNotifications
					{
						AuthenticationFailed = OnAuthenticationFailed
					}
				}
			);

			app.Map("/authenticate", (builder) =>
			{
				builder.Run(async (context) =>
				{
					var user = System.Web.HttpContext.Current.User;

					context.Response.Redirect("/");
					await Task.FromResult(true);
				});
			});

			app.Map("/login", (builder) =>
			{
				builder.Run(async (context) =>
				{
					if (HttpContext.Current.Request.IsAuthenticated)
					{
						context.Response.Redirect("/");
					}
					else
					{
						context.Authentication.Challenge(
								new AuthenticationProperties { RedirectUri = "/authenticate" },
								OpenIdConnectAuthenticationDefaults.AuthenticationType);
					}

					await Task.FromResult(true);
				});
			});

			app.Map("/logout", (builder) =>
			{
				builder.Run(async (context) =>
				{
					if (HttpContext.Current.Request.IsAuthenticated)
					{
						context.Authentication.SignOut(
								OpenIdConnectAuthenticationDefaults.AuthenticationType,
								CookieAuthenticationDefaults.AuthenticationType);
					}
					else
					{
						context.Response.Redirect("/");
					}

					await Task.FromResult(true);
				});
			});
		}

		/// <summary>
		/// Handle failed authentication requests by redirecting the user to the home page with an error in the query string
		/// </summary>
		/// <param name="context"></param>
		/// <returns></returns>
		private Task OnAuthenticationFailed(AuthenticationFailedNotification<OpenIdConnectMessage, OpenIdConnectAuthenticationOptions> context)
		{
			context.HandleResponse();
			context.Response.Redirect("/?errormessage=" + context.Exception.Message);

			return Task.FromResult(0);
		}
	}
}